"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formRenderer = void 0;

var _widjet = _interopRequireDefault(require("widjet"));

var _widjetUtils = require("widjet-utils");

var _renderers = require("./renderers");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var textContent = function textContent(n) {
  return n.innerText || n.textContent;
};

var jsonAttribute = function jsonAttribute(node, attr) {
  return JSON.parse(node.getAttribute(attr));
};

var jsonSourceAttribute = function jsonSourceAttribute(node, attr) {
  return JSON.parse(textContent(document.getElementById(node.getAttribute(attr))));
};

var jsonData = function jsonData(node, attr, sourceAttr) {
  return (node.hasAttribute(sourceAttr) ? jsonSourceAttribute(node, sourceAttr) : jsonAttribute(node, attr)) || {};
};

var formRenderer = (0, _widjetUtils.curry2)(function (options, data) {
  var schema = data.schema,
      values = data.values;
  var id = data.id || (0, _utils.getNextID)();
  var rootAttributePath = data.rootAttributePath || [];
  var renderers = options.renderers.map(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        p = _ref2[0],
        r = _ref2[1];

    return [p, r(function (a, b, c, d) {
      return renderObject(a, b, c, d);
    })];
  });
  var renderObject = (0, _renderers.objectRenderer)({
    id: id,
    fieldId: options.fieldId,
    fieldName: options.fieldName,
    renderField: (0, _widjetUtils.when)(renderers)
  });
  return options.formTemplate({
    id: id,
    content: renderObject(schema, values, rootAttributePath)
  });
});
exports.formRenderer = formRenderer;

_widjet.default.define('json-form', function (options) {
  var tpl = options.findTemplate || function (s) {
    return options["".concat(s, "Template")] || window.JST["json-form/".concat(s)];
  };

  var schemaSourceAttribute = options.schemaSourceAttribute || 'data-schema-source';
  var schemaAttribute = options.schemaAttribute || 'data-schema';
  var valueSourceAttribute = options.valueSourceAttribute || 'data-values-source';
  var valueAttribute = options.valueAttribute || 'data-values';
  var idAttribute = options.idAttribute || 'data-id';
  var DEFAULT_RENDERERS = [[(0, _renderers.typeIs)('object'), (0, _renderers.renderObjectField)(tpl('object'))], [(0, _renderers.typeIs)('array'), (0, _renderers.renderArrayField)(tpl('array'), tpl('arrayItem'))], [_widjetUtils.always, (0, _renderers.renderDefaultField)(tpl('field'), function (t) {
    return tpl(t);
  })]];
  var render = formRenderer({
    formTemplate: tpl('form'),
    fieldId: options.fieldId,
    fieldName: options.fieldName,
    renderers: (options.renderers || []).concat(DEFAULT_RENDERERS)
  });
  return function (container) {
    var id = container.getAttribute(idAttribute);
    var schema = jsonData(container, schemaAttribute, schemaSourceAttribute);
    var values = jsonData(container, valueAttribute, valueSourceAttribute);
    var rootAttributePath = container.dataset.rootAttributePath ? container.dataset.rootAttributePath.split(',') : [];
    container.appendChild((0, _widjetUtils.getNode)(render({
      schema: schema,
      values: values,
      id: id,
      rootAttributePath: rootAttributePath
    })));
    window.requestAnimationFrame(function () {
      return _widjet.default.dispatch(container, 'json-form:ready');
    });
  };
});