"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.renderDefaultField = exports.renderCompositeField = exports.renderArrayField = exports.renderObjectField = exports.objectRenderer = exports.typeIsComposite = exports.typeIs = void 0;

var _widjetUtils = require("widjet-utils");

var _utils = require("./utils");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var defaultFieldId = function defaultFieldId(id, path) {
  return [id].concat(path).join('-');
};

var typeIs = (0, _widjetUtils.curry2)(function (type, opts) {
  return opts.type === type;
});
exports.typeIs = typeIs;

var typeIsComposite = function typeIsComposite(opts) {
  return _typeof(opts.type) == 'object' && Array.isArray(opts.type);
};

exports.typeIsComposite = typeIsComposite;

var objectRenderer = function objectRenderer(_ref) {
  var fid = _ref.id,
      renderField = _ref.renderField,
      fieldName = _ref.fieldName,
      fieldId = _ref.fieldId;
  fieldName = fieldName || (0, _widjetUtils.apply)((0, _widjetUtils.inputName)());
  fieldId = fieldId || defaultFieldId;
  return function () {
    var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var values = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var objectName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    return (0, _utils.asTuple)(schema).map(function (_ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          key = _ref3[0],
          value = _ref3[1];

      var attributePath = objectName.concat(key);
      var name = fieldName(attributePath);
      var id = fieldId(fid, attributePath);

      var _getTypeAndParameters = (0, _utils.getTypeAndParameters)(value),
          _getTypeAndParameters2 = _slicedToArray(_getTypeAndParameters, 2),
          type = _getTypeAndParameters2[0],
          parameters = _getTypeAndParameters2[1];

      if (!type) {
        throw new Error("Field '".concat(key, "' is missing a 'type' key"));
      }

      return {
        id: id,
        type: type,
        parameters: parameters,
        name: name,
        attributePath: attributePath,
        attribute: key,
        value: values[key]
      };
    }).map(function (params) {
      return renderField(params);
    }).reduce(function (memo, str) {
      return memo + str;
    }, '');
  };
};

exports.objectRenderer = objectRenderer;
var renderObjectField = (0, _widjetUtils.curry3)(function (tpl, renderObject, opts) {
  return tpl((0, _widjetUtils.merge)(opts, {
    content: renderObject(opts.parameters.properties, opts.value, opts.attributePath)
  }));
});
exports.renderObjectField = renderObjectField;
var renderArrayField = (0, _widjetUtils.curry4)(function (tplUl, tplLi, renderObject, opts) {
  var itemsType = opts.parameters.items;

  var render = function render(i) {
    return renderObject(_defineProperty({}, i, itemsType), opts.value, opts.attributePath);
  };

  var renderItems = function renderItems(value, i) {
    return tplLi({
      content: render(i),
      index: i
    });
  };

  return tplUl((0, _widjetUtils.merge)(opts, {
    content: (opts.value || []).map(renderItems).join('')
  }));
});
exports.renderArrayField = renderArrayField;
var renderCompositeField = (0, _widjetUtils.curryN)(5, function (tplUl, tplLi, renderField, rO, opts) {
  var types = opts.type;

  var render = function render(type) {
    return renderField(rO, {
      type: type,
      value: opts.value,
      name: opts.name
    });
  };

  var renderTypes = function renderTypes(type, i) {
    return tplLi({
      content: render(type),
      index: i
    });
  };

  return tplUl((0, _widjetUtils.merge)(opts, {
    content: types.map(renderTypes).join('')
  }));
});
exports.renderCompositeField = renderCompositeField;
var renderDefaultField = (0, _widjetUtils.curry4)(function (tplWrapper, tplField, rO, opts) {
  return tplWrapper((0, _widjetUtils.merge)(opts, {
    content: tplField(opts.type)(opts)
  }));
});
exports.renderDefaultField = renderDefaultField;