"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNextID = getNextID;
exports.isJSON = isJSON;
exports.asTuple = asTuple;
exports.getTypeAndParameters = getTypeAndParameters;
exports.evaluateKey = evaluateKey;
var nextID = 0;

function getNextID() {
  return nextID++;
}

function isJSON(s) {
  return s.match(/^\{|^\[/);
}

function asTuple(object) {
  return Object.keys(object).map(function (key) {
    return [key, object[key]];
  });
}

function getTypeAndParameters(type) {
  var parameters = {};

  if (typeof type === 'string' && isJSON(type)) {
    type = JSON.parse(type);
  }

  if (type instanceof Object) {
    parameters = type;
    type = type.type;
  }

  return [type, parameters];
}

function evaluateKey(key, obj) {
  var keyPath = key.split('.');
  var localValue = obj;

  do {
    localValue = localValue[keyPath.shift()];
  } while (keyPath.length);

  return localValue;
}